API_VERSION = "v2"
API_ENDPOINT_PATH = f"/portal/api/external/{API_VERSION}"

LOGIN_CREDENTIAL = "/core/auth/create-access-token-by-credentials"
LOGIN_SERVICE = "/core/auth/create-access-token-by-service-account"



from enum import StrEnum
from pathlib import Path

import rich
import typer
from rich.progress import Progress
from rich.table import Table

from pydantic import RootModel

from ..enums import LoginProviderEnum
from ..schemas.models import ListSystemUsersElement
from ..schemas.requests import ListSystemUsersIn
from ..utils.models import UsersStats
from .utils import cli_init_api, user_login_info

app = typer.Typer()

state = {"env_file": None}


class OutputFormat(StrEnum):
    TABLE = "table"
    JSON = "json"


@app.callback()
def set_env_file(
    env_file: Path = typer.Option(
        Path("~/.pynta.toml"), "--env", "-e", help="Environment file in formato toml"
    ),
):
    state["env_file"] = env_file


@app.command("list")
def list_users(
    show_not_active: bool = typer.Option(False, help="Mostra anche gli utenti non attivi"),
    login_provider: list[LoginProviderEnum] = typer.Option(
        [], "--login-provider", "-lp", help="Filtra gli utenti con i login provider selezionati"
    ),
    show_no_login_provider: bool = typer.Option(
        False, help="Mostra solo gli utenti senza alcun login provider"
    ),
    user_filter: str = typer.Option(
        "",
        "--user-filter",
        "-uf",
        help="Filtra gli utenti con una ricerca fulltext su nome cognome e email",
    ),
    divergent_contact_email: bool = typer.Option(
        False,
        help="Mostra solo gli utenti che hanno email di contatto diverse da quelle"
        " del login provider",
    ),
    output_format: OutputFormat = typer.Option(
        OutputFormat.TABLE, "--out-format", "-of", help="Tipo di formato dell'output"
    ),
):
    filter_data = ListSystemUsersIn()
    if not show_not_active:
        filter_data.status_filter = [0]
    if login_provider:
        filter_data.login_provider_filter = list(login_provider)
    if show_no_login_provider:
        filter_data.login_provider_filter = []
    if user_filter:
        filter_data.full_text_filter = user_filter

    api, _ = cli_init_api(env_file=state["env_file"])
    with Progress() as progress:
        task = progress.add_task("Get data from Interacta...", total=None)
        users = api.all_users(data=filter_data)
        progress.remove_task(task)

    users_stats = UsersStats.create_by_user_list(users=users)

    if show_no_login_provider:
        users = users_stats.no_provider.values() if users_stats.no_provider else []

    if not users:
        rich.print("[red]Nessun risultato corrispondente ai parametri.[/red]")
        raise typer.Exit(2)

    if output_format == OutputFormat.TABLE:
        table = Table(
            "Cognome",
            "Nome",
            "Contatto Email",
            "Login Info",
            "Stato",
            show_lines=True,
        )
        for user in users:
            login_info = user_login_info(user=user)
            if divergent_contact_email and not login_info.startswith("[red]"):
                continue

            table.add_row(
                user.last_name,
                user.first_name,
                user.contact_email,
                login_info,
                "[green]attivo[/green]"
                if user.id in users_stats.active
                else "[red]non attivo[/red]",
            )

        if table.row_count == 0:
            rich.print("[red]Nessun risultato corrispondente ai parametri.[/red]")
            raise typer.Exit(3)

        rich.print(table)
        table_summary = Table("Dato", "Conteggio", show_header=False, title="Conteggi")
        for k, v in users_stats.counts.items():
            table_summary.add_row(str(k), str(v))
        rich.print(table_summary)
        raise typer.Exit(0)

    rich.print_json(RootModel[list[ListSystemUsersElement]](users).model_dump_json())



from typing import Any


class InteractaError(Exception):
    pass


class InteractaResponseError(InteractaError):
    def __init__(self, message: str, response: Any = None):
        super().__init__(message, response)
        self.message = message
        self.response = response


class InteractaLoginError(InteractaError):
    pass


class ObjectDoesNotFound(InteractaError):
    pass


class PostDoesNotFound(ObjectDoesNotFound):
    pass


class MultipleObjectsReturned(InteractaError):
    pass



# ref https://injenia.atlassian.net/wiki/spaces/IEAD/pages/3624075265/Autenticazione#Costruzione-del-jwtAssertion
SERVICE_AUTH_HEADERS = {
    # Sostituire con il valore "private_key_id" recuperato dal JSON delle credenziali
    # (Service Account). Attenzione che è un tipo JSON number, non una stringa.
    "kid": 0,
    "typ": None,
}

SERVICE_AUTH_PAYLOAD = {
    # jti - Sostituire con un identificativo univoco, esempio UUID
    "jti": "",
    # aud - Fisso al valore injenia/portal-authenticator
    "aud": "injenia/portal-authenticator",
    # iss - Sostituire con il valore "client_id" recuperato dal JSON delle credenziali
    # (Service Account)
    "iss": "",
    # iat- Unix timestamp dell'ora corrente
    "iat": "",
    # unit - Unix timestamp dell'ora di scadenza dell'assertion
    "exp": "",
}

PLAYGROUND_SETTINGS = {
    "base_url": "https://prod.development.lab.interacta.space",
    "auth_username": "interacta-test-api@interacta-prod",
    "auth_password": "MyInteractaPl@yground!",
    "playground_community": {
        "community_id": 1142,
    },
}



import json
from functools import cached_property
from pathlib import Path
from typing import Any

from pydantic import BaseModel, ConfigDict, HttpUrl, SecretStr, computed_field
from pydantic_settings_toml import TomlSettings, TomlSettingsError

from .enums import LoginProviderEnum
from .exceptions import InteractaError
from .models import ServiceAccountModel


class CommunitySettings(BaseModel):
    community_id: int = 0
    custom_fields: dict[str, int] = {}
    default_watcher_user_ids: list[int] | None = None

    @property
    def aliases(self):
        aliases = {}
        for attr, value in self.custom_fields.items():
            aliases[attr] = str(value)
        return aliases

    @property
    def custom_fields_id(self) -> list[int]:
        return list(self.custom_fields.values())


class InteractaUsersSettings(BaseModel):
    check_by_email: bool = True
    check_by_external_auth_service: bool = False
    external_login_providers: LoginProviderEnum | None = None


class ApiSettings(BaseModel):
    model_config = ConfigDict(validate_assignment=True)

    base_url: HttpUrl
    api_version: str = "v2"
    api_endpoint_path: str = "portal/api/external/"
    portal_path: str = "portal/"
    auth_service_file_path: Path | None = None
    auth_service_account: ServiceAccountModel | None = None
    auth_username: str | None = None
    auth_password: SecretStr | None = None

    def model_post_init(self, __context: Any) -> None:
        if self.auth_service_file_path:
            try:
                with open(self.auth_service_file_path) as f:
                    data = json.load(f)
                    self.auth_service_account = ServiceAccountModel(**data)
                return
            except Exception as exc:
                raise ValueError(f"Wrong service account settings: {exc}") from exc
        if not self.auth_username or not self.auth_password:
            raise ValueError("No authentication settings is set")

    @computed_field
    @cached_property
    def api_url(self) -> HttpUrl:
        url = HttpUrl(f"{self.base_url}{self.api_endpoint_path}{self.api_version}")
        return url

    @computed_field
    @cached_property
    def portal_url(self) -> HttpUrl:
        url = HttpUrl(f"{self.base_url}{self.portal_path}")
        return url


class InteractaSettings(ApiSettings):
    log_api_calls: bool = False
    operators_group_id: int | None = None
    users: InteractaUsersSettings | None = None


class AppSettings(TomlSettings):
    interacta: InteractaSettings | None = None


def init_app_settings(env_file: Path | None = None) -> AppSettings:
    try:
        return AppSettings(_env_file=env_file)
    except TomlSettingsError as tse:
        raise InteractaError(tse) from tse



from pathlib import Path

import rich
import typer
from rich.table import Table

from ..api import InteractaApi
from ..enums import LoginProviderEnum
from ..exceptions import InteractaError
from ..schemas.models import BaseListPostsElement, ListSystemUsersElement
from ..settings import AppSettings, InteractaSettings, init_app_settings


def cli_init_setting(env_file: Path | None = None) -> AppSettings:
    try:
        return init_app_settings(env_file=env_file)
    except InteractaError as ie:
        rich.print(f"[bold red]{ie}[/bold red]")
        raise typer.Exit(code=10) from ie


def cli_init_api(env_file: Path | None = None) -> tuple[InteractaApi, AppSettings]:
    settings = cli_init_setting(env_file=env_file)
    try:
        api = InteractaApi(settings=settings.interacta)
        api.login()
        return api, settings
    except InteractaError as ie:
        rich.print(f"[bold red]{ie}[/bold red]")
        raise typer.Exit(code=10) from ie


def table_list_posts(posts: list[BaseListPostsElement], settings: InteractaSettings) -> Table:
    table = Table("Id", "Titolo", "Descrizione", show_lines=True)
    for post in posts:
        table.add_row(
            f"[link={post.get_absolute_url(base_url=settings.portal_url)}]{post.id}[/link]",
            post.title,
            post.description_plain_text,
        )
    return table


def user_login_info(user: ListSystemUsersElement) -> str:
    results = (
        [LoginProviderEnum.CUSTOM.value] if LoginProviderEnum.CUSTOM in user.login_providers else []
    )
    for provider in [lp for lp in LoginProviderEnum if lp != LoginProviderEnum.CUSTOM]:
        provider_account_id = getattr(user, f"{provider}_account_id")
        if not provider_account_id:
            continue
        color = "green" if provider_account_id == user.contact_email else "red"
        out_provider = f"[{color}]{provider.value}: {provider_account_id} [/{color}]"
        results.append(out_provider)
    return "\n".join(results)



from pathlib import Path

import rich
import typer
from devtools import debug
from rich.table import Table
from rich.text import Text

from ..api import InteractaApi
from .users import app as users_app
from .utils import cli_init_setting, table_list_posts

app = typer.Typer(help="")
app.add_typer(users_app, name="users")

state = {"env_file": None}


@app.callback()
def set_env_file(
    env_file: Path = typer.Option(
        Path("~/.pynta.toml"), "--env", "-e", help="Environment file in formato toml"
    ),
):
    state["env_file"] = env_file


@app.command()
def get_community_definition(
    community_id: int = typer.Option(0, "--id", "-i"),
    community_name: str = typer.Option("", "--name", "-n"),
    fields_definition: bool = typer.Option(False, "--fields", "-fd"),
    workflow_definition: bool = typer.Option(False, "--workflow", "-wf"),
    output_table: bool = typer.Option(False, "--table", "-t"),
):
    """
    Mostra la definizione del post di una community e le relative impostazioni, dati workflow,
    campi custom ecc
    """
    if not community_id and not community_name:
        rich.print(
            "[bold red]Errore: inserisci almeno uno tra i due argomenti 'community_id' e"
            " 'community_name' [/bold red]"
        )
        raise typer.Exit(code=1)

    settings = cli_init_setting(env_file=state["env_file"]).interacta
    api = InteractaApi(settings=settings)
    api.login()

    if not community_id and community_name:
        if community_name not in settings.model_dump():
            rich.print(
                f"[bold red]Errore: non è presente alcuna configurazione per la community"
                f" '{community_name}'[/bold red]"
            )
            raise typer.Exit(code=1)
        community_id = settings.model_dump()[community_name]["community_id"]

    community = api.get_community_detail(community_id=community_id)
    community_name = community.community.name
    community_title = f"Community --> [cyan]{community_name}[/cyan]"
    post_definition = api.get_post_definition_detail(community_id=community_id)

    if fields_definition:
        if output_table:
            table = Table(
                "Id", "Label", "Type", "Enum Values", title=community_title, show_lines=True
            )
            for field in post_definition.field_definitions:
                table.add_row(
                    str(field.id),
                    field.label,
                    field.type.name,
                    ",".join([str((d.id, d.label)) for d in field.enum_values]),
                )
            rich.print(table)
        else:
            rich.print(f"{community_title}\n")
            for field in post_definition.field_definitions:
                rich.print(field)
        raise typer.Exit()

    if workflow_definition:
        if output_table:
            state_table = Table(
                "Id",
                "Nome",
                "Iniziale",
                "Finale",
                "Metadati",
                title=f"Stati di {community_title}",
                show_lines=True,
            )
            for state_data in post_definition.workflow_definition.states:
                if state_data.deleted:
                    continue

                state_color = (
                    state_data.metadata["style_color"]
                    if "style_color" in state_data.metadata
                    else ""
                )
                state_table.add_row(
                    str(state_data.id),
                    Text(state_data.name, style=f"bold {state_color}"),
                    str(state_data.init_state),
                    str(state_data.terminal),
                    str(state_data.metadata),
                )
            rich.print(state_table)
            transitions_table = Table(
                "Id",
                "Nome",
                "Trasizione",
                "Screen",
                title=f"Transizine di {community_title}",
                show_lines=True,
            )
            for transition in post_definition.workflow_definition.transitions:
                state_from_color = (
                    transition.from_state.metadata["style_color"]
                    if "style_color" in transition.from_state.metadata
                    else ""
                )
                state_to_color = (
                    transition.to_state.metadata["style_color"]
                    if "style_color" in transition.to_state.metadata
                    else ""
                )
                transition_txt = Text(transition.from_state.name, style=f"bold {state_from_color}")
                transition_txt.append(" --> ", style=f"bold {state_to_color}")
                transition_txt.append(transition.to_state.name, style=f"bold {state_to_color}")

                transitions_table.add_row(
                    str(transition.id), transition.name, transition_txt, str(transition.screen)
                )
            rich.print(transitions_table)
        else:
            rich.print_json(post_definition.workflow_definition.model_dump_json())
        raise typer.Exit()

    rich.print_json(post_definition.model_dump_json())


@app.command()
def list_posts(
    community: int = typer.Argument(..., help="Id della community"),
):
    """
    Lista dei post presenti in una community.
    """
    settings = cli_init_setting(env_file=state["env_file"]).interacta
    api = InteractaApi(settings=settings)
    api.login()

    posts = api.list_posts(community_id=community).items
    rich.print(table_list_posts(posts=posts, settings=api.settings))


@app.command()
def echo_settings():
    """
    Mostra gli environment
    """
    debug(cli_init_setting(env_file=state["env_file"]).model_dump())



from datetime import datetime
from datetime import datetime as type_datetime
from typing import Any

from pydantic import BaseModel, ConfigDict, EmailStr

from ..enums import FieldFilterTypeEnum, FieldTypeEnum
from ..exceptions import ObjectDoesNotFound
from .core import InteractaModel


class Link(BaseModel):
    label: str | None = None
    url: str | None = None


class AdminUserPreferences(InteractaModel):
    # AdminUserPreferencesDTO
    default_language_id: str | None = None
    default_timezone_id: int | None = None
    email_notifications_enabled: bool = True


class GoogleUserCredentialsConfiguration(InteractaModel):
    # GoogleUserCredentialsConfigurationDTO
    # Email autenticazione tramite servizi esterno Google
    google_account_id: str | None = None
    enabled: bool | None = None
    profile_photo_url: str | None = None


class MicrosoftUserCredentialsConfiguration(InteractaModel):
    # MicrosoftUserCredentialsConfigurationDTO
    # Email autenticazione tramite servizi esterno Microsoft
    microsoft_account_id: str | None = None
    enabled: bool | None = None
    profile_photo_url: str | None = None


class CustomUserCredentialsConfiguration(InteractaModel):
    # CustomUserCredentialsConfigurationDTO
    username: str | None = None
    can_user_manage_custom_credentials: bool | None = None
    active: bool | None = None
    profile_photo_url: str | None = None
    profile_photo: dict | None = None
    can_manage_profile_photo: bool | None = None


class UserCredentialsConfiguration(InteractaModel):
    # UserCredentialsConfigurationDTO
    google: GoogleUserCredentialsConfiguration | None = None
    microsoft: MicrosoftUserCredentialsConfiguration | None = None
    custom: CustomUserCredentialsConfiguration | None = None


class ResetUserCustomCredentialsCommand(InteractaModel):
    # ResetUserCustomCredentialsCommandDTO
    # Genera una password.
    generate_password: bool | None = None
    # Password da impostare. Campo da impostare nel caso in cui 'generatePassword' = false.
    password: list[str]
    # Indica se le credenziali devono essere cambiate al prossimo login.
    force_credentials_expiration: bool | None = None
    # Indica se è stata spedita un'email di notifica.
    email_notify_recipients: list[str]


class Language(InteractaModel):
    # LanguageDTO
    code: str | None = None
    description: str | None = None


class Timezone(InteractaModel):
    # TimezoneDTO
    id: int
    code: str | None = None
    zone_id: str | None = None
    description: str | None = None


class ZonedDatetime(InteractaModel):
    zoned_datetime: datetime | None = None
    local_datetime: datetime | None = None
    timezone: str | None = None


class GroupBase(InteractaModel):
    name: str | None = None
    email: str | None = None
    visible: bool | None = None
    external_id: str | None = None


class Group(GroupBase):
    # GroupDTO ##OK##
    id: int
    deleted: bool | None = None
    occ_token: int | None = None
    members_count: int | None = None


class ListSystemGroupsElement(Group):
    # ListSystemGroupsElementDTO
    pass


class UserBase(InteractaModel):
    id: int
    first_name: str | None = None
    last_name: str | None = None
    account_photo_url: str | None = None
    contact_email: EmailStr | None = None


class User(UserBase):
    # UserDTO ##OK##
    google_account_id: str | None = None
    microsoft_account_id: str | None = None
    service_account: bool | None = None
    system_account: bool | None = None
    active_dwd: bool | None = None
    deleted: bool | None = None
    blocked: bool | None = None
    external_id: str | None = None
    private_profile: bool | None = None
    licences: dict | None = None  # LicenseDTO


class UserInfoBase(InteractaModel):
    id: int
    name: str | None = None
    external_id: str | None = None


class UserInfoBusinessUnit(UserInfoBase):
    # UserInfoBusinessUnitDTO
    pass


class UserInfoArea(UserInfoBase):
    # UserInfoAreaDTO
    pass


class UserProfileInfo(UserBase):
    # UserProfileInfoDTO
    high_res_account_photo_url: str | None = None
    custom_photo_url: str | None = None
    google_photo_url: str | None = None
    microsoft_photo_url: str | None = None
    private_email: EmailStr | None = None
    private_email_verified: bool | None = None
    phone: str | None = None
    internal_phone: str | None = None
    mobile_phone: str | None = None
    place: str | None = None
    biography: str | None = None
    business_unit: UserInfoBusinessUnit | None = None
    area: UserInfoArea | None = None
    role: str | None = None
    manager: User | None = None
    employees: list[User] | None = None
    language: Language | None = None
    timezone: Timezone | None = None
    email_notifications_enabled: bool | None = None
    deleted: bool | None = None
    blocked: bool | None = None
    occ_token: int | None = None
    can_manage_profile_photo: bool | None = None
    edit_profile: bool | None = None
    private_profile: bool | None = None


class DONTUSEUserInfo(InteractaModel):
    # UserInfoDTO
    area: UserInfoArea | None = None
    business_unit: UserInfoBusinessUnit | None = None
    private_email: EmailStr | None = None
    private_email_verified: bool | None = None
    private_email_verification_required: bool | None = None


class UserInfo(InteractaModel):
    # UserInfoDTO
    area: UserInfoArea | None = None
    biography: str | None = None
    business_unit: UserInfoBusinessUnit | None = None
    internal_phone: str | None = None
    manager: User | None = None
    mobile_phone: str | None = None
    phone: str | None = None
    place: str | None = None
    role: str | None = None


class ListSystemUsersElement(User):
    # ListSystemUsersElementDTO
    login_providers: list[str] | None = None  # [ CUSTOM, GOOGLE, MICROSOFT ]
    last_access_timestamp: datetime | None = None
    user_profile_info: UserProfileInfo | None = None  # UserProfileInfoDTO


class UserFull(User):
    login_providers: list[str] | None
    last_access_timestamp: datetime | None
    user_profile_info: UserProfileInfo | None


class DriveAttachmentData(InteractaModel):
    # DriveAttachmentDataDTO
    drive_id: str = ""
    mime_type: str | None = None
    size: int | None = None
    web_view_link: str | None = None
    web_content_link: str | None = None


class TagBase(InteractaModel):
    id: int
    name: str | None = None
    deleted: bool | None = None


class Hashtag(TagBase):
    # HashtagDTO ##OK##
    external_id: str | None = None


class Tag(TagBase):
    # TagDTO
    visible: str | None = None


class ZonedDatetimeInput(InteractaModel):
    # ZonedDatetimeInputDTO
    datetime: type_datetime = None
    timezone: str | None = None


class PostAttachmentData(InteractaModel):
    # PostAttachmentDataDTO ##OK##
    id: int
    temporary_content_view_link: str | None = None
    temporary_content_download_link: str | None = None
    temporary_content_preview_image_link: str | None
    temporary_content_preview_image_animated_link: str | None = None
    temporary_content_preview_image_hi_res_link: str | None = None
    temporary_content_preview_image_hi_des_animated_link: str | None = None
    version_number: int | None = None
    type: int | None = None
    content_ref: str | None = None
    content_mime_type: str | None = None
    etag: str | None = None
    md5_hash: str | None = None
    size: int | None = None
    creator_user: User | None = None
    creation_timestamp: int | None = None
    hashtags: list[Hashtag] = []
    reference: bool = False
    drive: dict | None = None


class PostWorkflowDefinitionState(InteractaModel):
    #  PostWorkflowDefinitionStateDTO
    id: int
    init_state: bool | None = None
    name: str = ""
    metadata: dict | None = None
    terminal: bool | None = None
    deleted: bool | None = None


class Post(InteractaModel):
    # ROOT
    id: int = 0
    community_id: int = 0
    custom_id: str | None = None
    title: str = ""
    description_plain_text: str | None = None
    description_delta: str | None = None
    visibility: int | None = None
    announcement: bool | None = None
    custom_data: Any | None = None
    creator_user: User | None = None
    creation_timestamp: datetime | None = None
    last_modify_user: User | None = None
    last_modify_timestamp: datetime | None = None
    last_operation_timestamp: datetime | None = None
    main_attachment: PostAttachmentData | None = None
    attachments_count: int | None = None
    image_attachments_count: int | None = None
    video_attachments_count: int | None = None
    comments_count: int | None = None
    views_count: int | None = None
    viewed_by_users_count: int | None = None
    show_like_section: bool | None = None
    likes_count: int | None = None
    liked_by_me: bool | None = None
    tasks_count: int | None = None
    followed_by_me: bool | None = None
    total_standard_tasks_count: int | None = None

    def get_absolute_url(self, base_url: str | None = None):
        url = f"post/{self.id}"
        if base_url:
            url = f"{base_url}{url}"
        return url


class BaseListPostsElement(Post):
    # BaseListPostsElementDTO
    workflow_state_description: str | None = None
    workflow_state_color: str | None = None
    watchers_count: int | None = None
    viewed_by_me: bool | None = None
    capabilities: dict | None = None  # GetPostCapabilitiesResponseDTO
    cover_image: dict | None = None  # PostCoverImageDTO


class PostDetail(Post):
    # PostDetailDTO
    current_workflow_state: PostWorkflowDefinitionState | None = None
    current_workflow_screen_data: dict | None = None
    attachments: list[dict] | None = None  # PostAttachmentDataDTO
    mentions: list[User] | None = None
    watchers: list[User] | None = None
    watcher_users: list[User] | None = None
    watcher_groups: list[Group] | None = None
    hashtags: list[Hashtag] | None = None  # HashtagDTO
    cover_image: dict | None = None  # PostCoverImageDTO
    draft: bool = False  # Creazione in stato bozza/pubblicato
    draft_data: dict | None = None  # DraftPostDataDTO
    scheduled_publication: ZonedDatetimeInput | None = None
    # Info stato del post programmato [1=SUCCESS, 2=FAILED]
    scheduled_publication_result: int | None = None


class DriveAttachment(InteractaModel):
    drive_id: str = ""
    mime_type: str | None = None
    size: int | None = None
    web_view_link: str | None = None
    web_content_link: str | None = None


class Hashtag(InteractaModel):
    id: int
    name: str | None = None
    community_id: int | None = None
    external_id: str | None = None
    deleted: bool | None = None


class EnumValue(InteractaModel):
    # PostFieldConfigEnumValueDTO
    id: int
    label: str | None = None
    external_id: str | None = None
    parent_ids: list[int] | None = None
    deleted: bool | None = None


class FieldMetadata(InteractaModel):
    model_config = ConfigDict(extra="allow")

    catalog_id: int | None = None
    hierarchical: bool | None = None
    generic_entity_list_config_id: str | None = None
    community_id: int | None = None


class FieldValidation(InteractaModel):
    # PostFieldValidationDTO
    id: int
    field_id: int | None = None
    validation_type_id: int | None = None
    parameter: str | None = None
    order: int | None = None


class FieldBase(InteractaModel):
    id: int
    name: str | None = None
    label: str | None = None
    type: FieldTypeEnum | None = None
    required: bool | None = None
    searchable: bool | None = None
    sortable: bool | None = None
    enum_values: list[EnumValue] | None = None
    metadata: FieldMetadata | None = None

    @property
    def catalog_id(self):
        return self.metadata.catalog_id

    def get_enum_id(self, label: str) -> int | None:
        for option in self.enum_values:
            if label == option.label:
                return option.id
        raise ValueError(f"No enum value with label {label}")


class FieldDefinition(FieldBase):
    # PostFieldDefinitionDTOPostFieldDefinitionDTO
    description: str | None = None
    parent_id: int | None = None
    readonly: bool | None = None
    visible_in_preview: bool | None = None
    visible_in_detail: bool | None = None
    visible_in_create: bool | None = None
    visible_in_edit: bool | None = None
    external_id: str | None = None
    validations: list[FieldValidation] | None = None


class PostWorkflowDefinitionScreenFieldAssociation(FieldBase):
    # PostWorkflowDefinitionScreenFieldAssociation
    pass


class WorkflowDefinitionScreen(InteractaModel):
    # WorkflowDefinitionScreenDTO
    id: int
    name: str | None = None
    message: str | None = None
    field_metadatas: list[PostWorkflowDefinitionScreenFieldAssociation] | None = None


class PostWorkflowDefinitionState(InteractaModel):
    # PostWorkflowDefinitionStateDTO
    id: int
    init_state: bool | None = None
    name: str | None = None
    metadata: dict | None = None
    terminal: bool | None = None
    deleted: bool | None = None


class PostWorkflowDefinitionScreenField(InteractaModel):
    # PostWorkflowDefinitionScreenFieldDTO
    id: int
    label: str | None = None
    type: FieldTypeEnum | None = None
    parent_id: int | None = None
    readonly: bool | None = None
    searchable: bool | None = None
    sortable: bool | None = None
    visible_in_create: bool | None = None
    visible_in_edit: bool | None = None
    external_id: str | None = None
    enum_values: list[EnumValue] | None = None
    metadata: FieldMetadata | None = None


class PostWorkflowDefinitionTransition(InteractaModel):
    # PostWorkflowDefinitionTransitionDTO
    id: int
    from_state: PostWorkflowDefinitionState | None = None
    to_state: PostWorkflowDefinitionState | None = None
    name: str | None = None
    metadata: dict | None = None
    screen: WorkflowDefinitionScreen | None = None


class PostWorkflowDefinition(InteractaModel):
    # PostWorkflowDefinitionDTO
    id: int
    title: str | None = None
    states: list[PostWorkflowDefinitionState] | None = None
    transitions: list[PostWorkflowDefinitionTransition] | None = None
    screen_field_metadatas: list[PostWorkflowDefinitionScreenField] | None = None
    empty: bool | None = None

    def get_state(self, name: str) -> PostWorkflowDefinitionState:
        for state in self.states:
            if state.name.lower() == name.lower():
                return state
        raise ObjectDoesNotFound(f"State {name} not found")

    def get_transition(
        self, from_state_id: int, to_state_id: int
    ) -> PostWorkflowDefinitionTransition:
        for transition in self.transitions:
            if from_state_id == transition.from_state.id and to_state_id == transition.to_state.id:
                return transition
        raise ObjectDoesNotFound(
            f"Transition from state {from_state_id} to state {to_state_id} not found"
        )

    def get_screen_field(self, label: str) -> PostWorkflowDefinitionScreenField:
        for screen_field in self.screen_field_metadatas:
            if screen_field.label.lower() == label.lower():
                return screen_field
        raise ObjectDoesNotFound(f"Screen field metadata {label} not found")


class PostDefinition(InteractaModel):
    acknowledge_task_enabled: bool | None = None
    attachment_enabled: bool | None = None
    attachment_max_size: int | None = None
    community_id: int
    community_relations: list[dict] | None = None  # creare model
    custom_fields_enabled: bool | None = None
    default_post_visibility: int | None = None
    description_enabled: int | None = None
    field_definitions: list[FieldDefinition] | None = None
    workflow_definition: PostWorkflowDefinition | None = None  # creare model
    title_enabled: int | None = None
    watchers_enabled: int | None = None
    watchers_visible_in_preview: bool | None = None
    manual_author_enabled: bool | None = None
    hash_tag_enabled: bool | None = None
    like_enabled: bool | None = None
    offline_Enabled: bool | None = None
    hashtags: list[Hashtag] | None = None
    post_views: list[dict] | None = None  # creare model
    inverse_community_relations: list[dict] | None = None  # creare model

    def get_field_by_id(self, field_id: int) -> FieldDefinition | None:
        for field in self.field_definitions:
            if field_id == field.id:
                return field
        raise ObjectDoesNotFound(f"Field with id '{field}' not found in field_definitions")

    def get_enum_id(self, field_id: int, label: str) -> int | None:
        field = self.get_field_by_id(field_id=field_id)
        return field.get_enum_id(label=label)

    def get_field_by_label(self, field_label: str) -> FieldDefinition | None:
        for field in self.field_definitions:
            if field_label.lower() == field.label.lower():
                return field
        raise ObjectDoesNotFound(f"Field with id '{field}' not found in field_definitions")

    @property
    def custom_fields_ids(self) -> list[int]:
        return [int(field.id) for field in self.field_definitions]

    @property
    def catalog_ids(self) -> list[int]:
        return [field.catalog_id for field in self.field_definitions if field.catalog_id]


class PostEditableContentData(InteractaModel):
    title: str = ""
    description: str | None = ""
    description_delta: str | None = None
    visibility: int | None = None
    custom_data: dict | None = None
    current_workflow_screen_data: dict | None = None
    attachments: list[dict] | None = None
    mentions: list[User] | None = None
    watchers: list[User] | None = None
    watchers_users: list[User] | None = None
    watchers_groups: list[Group] | None = None
    hashtags: list[Hashtag] | None = None
    draft: bool | None = None
    draft_data: dict | None = None
    scheduled_publication: dict | None = None
    scheduled_publication_result: int | None = None


class Community(InteractaModel):
    # -- GetCommunityDetailsResponseDTO
    draft: bool | None = None  # Se true community non ancora pubblicata.
    admin_capabilities: dict | None = None
    operational_capabilities: dict | None = None
    # Link temporaneo download immagine di copertina della community.
    cover_image_temporary_content_view_link: str | None = None
    name: str | None = None  # Nome della community.
    workspace_id: int | None = None  # Id workspace di appartenenza.
    color_code: str | None = None  # Colore community.
    etag: int | None = None  # ETag
    members_count: int | None = None  # Numero membri community.
    creation_timestamp: int | None = None  # Istante creazione
    creator_user: User | None = None
    id: int | None = None  # Identificativo univoco della community.
    description: str | None = None  # Descrizione della community.


class CustomFieldFilter(InteractaModel):
    # CustomFieldFilterDTO
    # Identificativo univoco del campo del post.
    column_id: int | None = None
    # ipo di ricerca [1=EQUAL, 2=INTERVAL, 3=LIKE, 4=IN, 5=CONTAINS, 6=IS_NULL_OR_IN, 7=IS_EMPTY].
    type_id: FieldFilterTypeEnum | None = None
    # Parametri del filtro, ove necessari.
    parameters: list[Any] | None = None


class AcknowledgeTaskFilter(InteractaModel):
    # AcknowledgeTaskFilterDTO
    confirmed: bool | None = None
    assigned_to_me: bool | None = None


class CatalogEntry(InteractaModel):
    # CatalogEntryDTO
    id: int
    catalog_id: int
    label: str | None = None
    external_id: str | None = None
    parent_ids: list[int] | None = None
    deleted: bool = False


class Catalog(InteractaModel):
    # CatalogDTO
    id: int
    name: str
    etag: int | None = None
    paged: bool = False
    entries: list[CatalogEntry] | None = None


class PostCommentAttachment(InteractaModel):
    # PostCommentAttachmentDTO
    id: int
    temporary_content_view_link: str | None = None
    temporary_content_download_link: str | None = None
    temporary_content_preview_image_link: str | None = None
    temporary_content_preview_image_animated_link: str | None = None
    temporary_content_preview_image_hi_res_link: str | None = None
    temporary_content_preview_image_hi_res_animated_link: str | None = None
    name: str | None = None
    content_ref: str | None = None
    content_mime_type: str | None = None
    etag: str | None = None
    md5_hash: str | None = None
    size: int | None = None
    creator_user: User | None = None
    creation_timestamp: datetime | None = None


class PostComment(InteractaModel):
    # PostCommentDTO
    id: int
    comment_delta: str | None = None
    comment_plain_text: str | None = None
    creator_user: User | None = None
    creation_timestamp: datetime | None = None
    mentions: list[User] | None = None
    attachments: list[PostCommentAttachment] | None = None  # PostCommentAttachmentDTO
    deleted: bool | None | None
    parent_comment: int | None = None
    likes_count: int | None = None
    liked_by_me: bool | None | None
    show_like_section: bool | None | None



import functools
import json
import logging

import requests
from jwt.exceptions import InvalidTokenError
from requests import Response

from pydantic import BaseModel

from .exceptions import InteractaResponseError
from .schemas.models import InteractaModel
from .settings import ApiSettings

logger = logging.getLogger(__name__)


class Api:
    def __init__(self, settings: ApiSettings):
        self.access_token = None
        self._log_calls = False
        self.settings = settings

    def call_post(
        self, path: str, params: dict = None, headers: dict = None, data: dict | str = None
    ):
        return self.call_api(
            "post", path=path, params=params, headers=headers, data=prepare_data(data)
        )

    def call_get(self, path: str, params: str = None, headers: dict = None):
        return self.call_api("get", path=path, params=params, headers=headers)

    def call_put(
        self, path: str, params: str = None, headers: dict = None, data: dict | str = None
    ):
        return self.call_api(
            "put", path=path, params=params, headers=headers, data=prepare_data(data)
        )

    def call_delete(self, path: str, headers: dict = None, **kwargs):
        return self.call_api("delete", path=path, headers=headers, **kwargs)

    def call_api(
        self,
        method: str,
        path: str,
        params: dict | None = None,
        headers: dict | None = None,
        data: dict | str | None = None,
        **kwargs,
    ):
        url = f"{self.settings.api_url}{path}"
        request_method = getattr(requests, method)
        if self._log_calls:
            log_msg = f"API CALL: URL [{url}] HEADERS [{headers}] DATA [{data}]"
            logger.info(log_msg)
        response = request_method(url, headers=headers, data=data, params=params, **kwargs)
        if response.status_code != 200:
            raise InteractaResponseError(format_response_error(response), response=response)
        return response


def mock_validate_kid(self, kid) -> None:
    if not isinstance(kid, str) and not isinstance(kid, int):
        raise InvalidTokenError("Key ID header parameter must be a string or an int")


def prepare_data(data=None):
    if not data:
        return json.dumps({})
    if isinstance(data, BaseModel):
        if isinstance(data, InteractaModel):
            data = data.model_dump_json(by_alias=True)
        else:
            data = data.model_dump_json()
    else:
        data = json.dumps(data)
    return data.encode("utf-8")


def interactapi(func=None, *, schema_out=None):
    if func is None:
        return functools.partial(interactapi, schema_out=schema_out)

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        if "headers" not in kwargs:
            kwargs["headers"] = self.authorized_header
        else:
            kwargs["headers"].update(self.authorized_header)
        response = func(self, *args, **kwargs)
        if not schema_out:
            return response
        result = schema_out.model_validate(response.json())
        result._response = response
        return result

    return wrapper


def format_response_error(response: Response) -> str:
    return (
        f"url: {response.url} - response {response.status_code}"
        f" headers: {response.headers} content: {response.text}"
    )



from typing import Literal

from pydantic import BaseModel, ConfigDict, EmailStr

from .core import InteractaIn, InteractaModel, PaginatedIn
from .models import (
    AcknowledgeTaskFilter,
    AdminUserPreferences,
    CustomFieldFilter,
    DriveAttachmentData,
    GroupBase,
    ResetUserCustomCredentialsCommand,
    UserCredentialsConfiguration,
    UserInfo,
    ZonedDatetimeInput,
)


class AttachmentIn(BaseModel):
    # EditCustomPostRequestDTO
    # InputPostAttachmentDTO
    type: int | None = None
    attachment_id: int | None = None
    name: str | None = None
    content_ref: str | None = None  # Il riferimento sul cloudStorage dell'allegato.
    referenced_attachment_id: int | None = None  # Identificativo dell'allegato di riferimento
    hashtag_ids: list[int] | None = None
    drive: DriveAttachmentData | None = None


class CustomPostIn(InteractaModel):
    # ROOT
    title: str
    description: str | None = None  # Descrizione del post
    description_format: int = (
        1  # Formato della descrizione del post, facoltativo (1=delta, 2=markdown, default: 1)
    )
    custom_data: dict | None = None  # Dati custom del post
    # Formato dei campi custom di tipo 11-delta, facoltativo (1=delta, 2=plain text, default: 1)
    delta_area_format: int = 1
    visibility: int | None = None
    # Identificativo dello stato iniziale del workflow, se la community lo permette
    workflow_init_state_id: int | None = None
    draft: bool = False  # Creazione in stato bozza/pubblicato
    scheduled_publication: ZonedDatetimeInput | None = None


class CreateCustomPostIn(CustomPostIn):
    # CreateCustomPostRequest
    attachments: list[AttachmentIn] | None = None
    watcher_user_ids: list[int] | None = None
    announcement: bool | None = None
    client_uid: str | None = None


class EditCustomPostIn(CustomPostIn):
    # EditCustomPostRequestDTO
    add_attachments: list[AttachmentIn] = []
    update_attachments: list[AttachmentIn] = []
    remove_attachment_ids: list[int] = []
    add_watcher_user_ids: list[int] = []
    remove_watcher_user_ids: list[int] = []


class ListCommunityPostsIn(InteractaIn):
    # ListCommunityPostsRequestDTO
    # Filtro sul titolo
    title: str | None = None
    # Filtro sulla descrizione
    description: str | None = None
    # Filtro utente creatore
    created_by_user_id: int | None = None
    # Filtro lowerbound sulla data di creazione
    creation_timestamp_from: int | None = None
    # Filtro upperbound sulla data di creazione
    creation_timestamp_to: int | None = None
    # Filtro utente ultima modifica
    modified_by_user_id: int | None = None
    # Filtro lowerbound sulla data di modifica
    modified_timestamp_from: int | None = None
    # Filtro upperbound sulla data di modifica
    modified_timestamp_to: int | None = None
    # Contenuto del post
    contains_text: str | None = None
    # Filtro post seguiti da me
    followed_by_me: bool | None = None
    # Filtro post per cui l'utente ha qualcosa da fare
    to_manage: bool | None = None
    # Filtro post per cui l'utente può effettuare una transizione di stat
    to_manage_processes: bool | None = None
    # Filtro post per cui l'utente ha dei task di tipo STANDARD non evase (scadute o da fare)
    # assegnate
    to_manage_standard_tasks: bool | None = None
    # Filtro post per cui l'utente è stato menzionato
    mentioned: bool | None = None
    # Stato del workflow del post (parametro passato per identificativo)
    current_workflow_status_ids: list[int] | None = None
    # Filtro sulla tipoloiga del post [1=CUSTOM, 2=EVENTO].
    post_type: int | None = None
    # Filtro sulla tipoloiga del post [1=CUSTOM, 2=EVENTO]
    post_types: list[int] | None = None
    # Filtro per le bozze [null=nessun filtro, 1=STANDARD_DRAFT, 2=SCHEDULED_DRAFT
    draft_type: int | None = None
    event_post_filter: dict | None = None
    # Criterio di ordinamento. Valori ammessi ['customId', 'title', 'creatorUser',
    # 'creationTimestamp', 'lastModifyUser', 'lastModifyTimestamp', 'lastModifyAndCommentTimestamp',
    #  'viewedByMeTimestamp', 'modifiedByMeTimestamp', 'commentedByMeTimestamp',
    # 'scheduled_publication', 'recency', 'customField-{id del campo custom}'].
    order_by: str | None = None
    # Filtri sugli hashtag
    hashtag_ids: list[int] | None = None
    # Indica se i filtri sugli hashtag vanno combinati in AND o in OR (default = false)
    hashtags_logical_and: bool | None = None
    # Filtri sugli hashtag degli allegati
    attachment_hashtag_ids: list[int] | None = None
    # Indica se i filtri sugli hashtag degli allegati vanno combinati in AND o in OR
    #  (default = false)
    attachment_hashtags_logical_and: bool | None = None
    # Filtro sulla visibilità pubblica/privata
    visibility: int | None = None
    custom_field_filters: list[CustomFieldFilter] | None = None
    # Filtri sui campi custom
    post_field_filters: list[CustomFieldFilter] | None = None
    # Filtri sui campi screen
    screen_field_filters: list[CustomFieldFilter] | None = None
    acknowledge_task_filter: AcknowledgeTaskFilter | None = None
    # Filtro per le bozze
    draft: bool | None = None
    # Ordinamento sui post pinnati. Default: 'true'
    pinned_first: bool | None = None


class CommunityPostFilters(InteractaModel):
    # 	CommunityPostFiltersDTO
    # Filtro sul titolo
    title: str | None = None
    # Filtro sulla descrizione
    description: str | None = None
    # Filtro utente creatore
    created_by_user_id: int | None = None
    # Filtro utenti creatori
    created_by_user_ids: list[int] | None = None
    # Filtro gruppi di utenti creatori
    created_by_group_ids: list[int] | None = None
    # Filtro lowerbound sulla data di creazione
    creation_timestamp_from: int | None = None
    # Filtro upperbound sulla data di creazione
    creation_timestamp_to: int | None = None
    # Filtro utente ultima modifica
    modified_by_user_id: int | None = None
    # Filtro lowerbound sulla data di modifica
    modified_timestamp_from: int | None = None
    # Filtro upperbound sulla data di modifica
    modified_timestamp_to: int | None = None
    # Contenuto del post
    contains_text: str | None = None
    # Filtro post seguiti da me
    followed_by_me: bool | None = None
    # Filtro post per cui l'utente ha qualcosa da fare
    to_manage: bool | None = None
    # Filtro post per cui l'utente può effettuare una transizione di stat
    to_manage_processes: bool | None = None
    # Filtro post per cui l'utente ha dei task di tipo STANDARD non evase (scadute o da fare)
    # assegnate
    to_manage_standard_tasks: bool | None = None
    # Filtro post per cui l'utente è stato menzionato
    mentioned: bool | None = None
    # Stato del workflow del post (parametro passato per identificativo)
    current_workflow_status_ids: list[int] | None = None
    # Filtro sulla tipoloiga del post [1=CUSTOM, 2=EVENTO, 3=QUESTIONARIO]
    post_types: list[int] | None = None
    # Filtro sulla tipoloiga del post [1=QUESTIONARIO, 2=MISSIONE]
    post_survey_types: list[int] | None = None
    # Filtro per le bozze [null=nessun filtro, 1=STANDARD_DRAFT, 2=SCHEDULED_DRAFT
    draft_type: int | None = None
    event_post_filter: dict | None = None  # TODO --> EventPostFilterDTO
    # Filtro per recuperare solo post pinnati
    only_pinned: bool | None = None
    # Filtro sull'ID del post (utile per visualizzare tutti gli attachments di un singolo post).
    post_id_: int | None = None
    # Filtri sugli hashtag
    hashtag_ids: list[int] | None = None
    # Indica se i filtri sugli hashtag vanno combinati in AND o in OR (default = false)
    hashtags_logical_and: bool | None = None
    # Filtro sulla visibilità pubblica/privata
    visibility: int | None = None
    # Filtri sui campi custom
    post_field_filters: list[CustomFieldFilter] | None = None
    # Filtri sui campi screen
    screen_field_filters: list[CustomFieldFilter] | None = None
    acknowledge_task_filter: AcknowledgeTaskFilter | None = None
    # Filtro per le bozze
    draft: bool | None = None


class CommunityAttachmentFilters(InteractaModel):
    # CommunityAttachmentFiltersDTO
    # Filtro sul nome
    name: str | None = None
    # Filtro utente creatore
    created_by_user_id: int | None = None
    # Filtro utenti creatori
    created_by_user_ids: list[int] | None = None
    # Filtro gruppi di utenti creatori
    created_by_group_ids: list[int] | None = None
    # Filtro sul mimeType Valori ammessi: [1-AUDIO, 2-IMAGE, 3-VIDEO, 4-PDF, 5-TEXT, 6-DOCUMENT,
    # 7-SPREADSHEET, 8-PRESENTATION, 9-DRAWING, 10-ARCHIVE]
    mime_types: list[int] | None = None
    # Filtro sulla categoria di mimeType. Valori ammessi: ['multimedia', 'other']
    mime_type_category: Literal["multimedia", "other"]
    # Filtro sul tipo degli allegati. Valori ammessi: 1 (STORAGE, default), 2 (DRIVE)
    types: list[int] | None = None
    # Filtro per escludere i file provenienti da filePicker
    exclude_file_pickers: bool | None = None
    # Filtro sul tipo dell'entità dell'allegato. Valori ammessi:
    # 1 (POST, default), 2 (TASK), 3 (COMMENT)
    entity_types: list[int]
    # Filtro lowerbound sulla data di creazione
    creation_timestamp_from: int | None = None
    # Filtro upperbound sulla data di creazione
    creation_timestamp_to: int | None = None
    # Filtro utente ultima modifica
    modified_by_user_id: int | None = None
    # Filtro lowerbound sulla data di modifica
    modified_timestamp_from: int | None = None
    # Filtro upperbound sulla data di modifica
    modified_timestamp_to: int | None = None
    # Filtro sull'ID del filePicker di un campo custom (utile per visualizzare gli attachments
    # appartenenti ad uno specifico campo custom file picker).
    post_file_picker_field_id: int | None = None
    # Filtro sull'ID del filePicker di un campo screen del workflow (utile per visualizzare gli
    # attachments appartenenti ad uno specifico campo screen file picker).
    wf_screen_file_picker_field_id: int | None = None
    # Filtro sugli hashtag
    hashtag_ids: list[int] | None = None
    # Indica se i filtri sugli hashtag vanno combinati in AND o in OR (default = false)
    hashtags_logical_and: bool | None = None


class ListCommunityPostsFilteredIn(InteractaIn):
    # ListCommunityPostsFilteredRequestDTO
    community_post_filters: CommunityPostFilters | None = None
    community_attachment_filters: CommunityAttachmentFilters | None = None
    # Criterio di ordinamento. Valori ammessi ['postCustomId', 'postTitle', 'postCreatorUser',
    # 'postCreationTimestamp', 'postLastModifyUser', 'postLastModifyTimestamp',
    # 'postLastModifyAndCommentTimestamp', 'postViewedByMeTimestamp', 'postModifiedByMeTimestamp',
    # 'postCommentedByMeTimestamp', 'postScheduledPublication', 'postRecency',
    # 'postCustomField-{id del campo custom}'].
    order_by: str | None = None
    # Ordinamento sui post pinnati. Default: 'true'
    pinned_first: bool | None = None


### Workflow


class EditPostWorkflowScreenDataIn(InteractaModel):
    # EditPostWorkflowScreenDataRequestDTO
    model_config = ConfigDict(validate_assignment=True)

    screen_data: dict = {}
    delta_area_format: int = 1


class ExecutePostWorkflowOperationIn(EditPostWorkflowScreenDataIn):
    # ExecutePostWorkflowOperationRequestDTO
    screen_occ_token: int


### Utenti


class ListSystemUsersIn(InteractaIn):
    # ListSystemUsersRequestDTO

    # Filtro sul tipo di capability che deve avere l'utente, i valori ammessi sono:
    # [manageUsers, manageDigitalWorplace]
    admin_capabilities: list[Literal["manageUsers", "manageDigitalWorplace"]] | None = None
    # Filtro fulltext su nome cognome e email
    full_text_filter: str | None = None
    # Filtro sul nome dell'utente
    first_name_prefix_full_text_filter: str | None = None
    # Filtro sul cognome dell'utente
    last_name_prefix_full_text_filter: str | None = None
    # Filtro sull'email dell'utente
    email_prefix_full_text_filter: str | None = None
    # Filtro sull'email di autenticazione con servizi esterni
    external_auth_service_email_full_text_filter: str | None = None
    # Filtro sullo stato dell'utente, 0 = solo utenti attivi
    status_filter: list[int] | None = None
    # Filtro sui workspace
    workspace_ids: list[int] | None = None
    # Filtro sulle community
    community_ids: list[int] | None = None
    # Filtro per businessUnits
    business_unit_ids: list[int] | None = None
    # Filtro per area
    area_ids: list[int] | None = None
    # Filtro full-text per luogo
    place: str | None = None
    # Filtro per data di creazione
    creation_timestamp_from: int | None = None
    # Filtro per data di creazione
    creation_timestamp_to: int | None = None
    # Filtro per data di creazione
    last_access_timestamp_from: int | None = None
    # Filtro per data di creazione
    last_access_timestamp_to: int | None = None
    # Filtro per riferimento esterno
    external_id_full_text_filter: str | None = None
    # Filtro sull'email dell'utente
    personal_email_full_text_filter: str | None = None
    # Filtro per ruolo
    role: str | None = None
    # Filtro per manager
    manager_ids: list[int] | None = None
    # Filtro per lingua
    lang: list[str] | None = None
    # Filtro per fuso orario
    time_zone_ids: list[int] | None = None
    # Filtro per tipologia login
    login_provider_filter: list[str] | None = None
    # Filtro Sezione Persone Abilitata
    people_section_enabled: bool | None = None
    # Filtro Visibile nella Sezione Persone
    visible_in_people_section: bool | None = None
    # Filtro Profilo Ridotto (Base/Avanzato)
    reduced_profile: bool | None = None
    # Filtro Visualizzazione Altri Profili Abilitata
    view_user_profiles: bool | None = None
    # Id campo di ordinamento
    order_type_id: str | None = None


class UserSettingsIn(InteractaModel):
    # UserSettingsRequestDTO
    # Sezione Persone abilitata/disabilitata
    people_section_enabled: bool = True
    # Utente visibile nella sezione persone
    visible_in_people_section: bool = True
    # Visualizzazione profilo in versione ridotta/estesa
    reduced_profile: bool = False
    # Possibilità di visualizzare o meno il profilo di utenti terzi
    view_user_profiles: bool = True


class DataUserInfoIn(InteractaModel):
    id: int


class UserInfoIn(UserInfo):
    business_unit: DataUserInfoIn | None = None
    area: DataUserInfoIn | None = None
    manager: DataUserInfoIn | None = None


class UserEditBase(InteractaModel):
    firstname: str = ""
    lastname: str = ""
    contact_email: EmailStr | None = None
    private_email: EmailStr | None = None
    external_id: str | None = None
    user_preferences: AdminUserPreferences | None = None
    user_info: UserInfoIn | None = None
    user_settings: UserSettingsIn | None = None


class UserIn(UserEditBase):
    pass


class CreateUserIn(UserIn):
    # CreateUserRequestDTO
    user_credentials_configuration: UserCredentialsConfiguration | None = None
    reset_user_custom_credentials_command: ResetUserCustomCredentialsCommand | None = None


class EditUserIn(UserIn):
    # EditUserRequestDTO
    occ_token: int | None = None


### Gruppi


class ListSystemGroupsIn(InteractaIn):
    # ListSystemGroupsRequestDTO

    # Filtro sul tipo di capability che deve avere l'utente, i valori ammessi sono:
    # [manageUsers, manageDigitalWorplace]
    admin_capabilities: list[Literal["manageUsers", "manageDigitalWorplace"]] | None = None
    # Filtro fulltext su nome e email
    full_text_filter: str | None = None
    # Filtro sullo stato dell'utente
    status_filter: list[int] | None = None  # 0 == solo gruppi non eliminati
    # Filtro sui workspace
    workspace_ids: list[int] | None = None
    # Filtro per escludere i gruppi che contengono l'utente
    exclude_group_by_member_user_id: int | None = None
    # Id campo di ordinamento
    order_type_id: str | None = None


class ListGroupMembersIn(InteractaIn):
    # ListGroupMembersRequestDTO
    pass


class CreateGroupIn(GroupBase):
    # CreateGroupRequestDTO
    member_ids: list[int] | None = None


class EditGroupIn(CreateGroupIn):
    # EditGroupRequestDTO
    occ_token: int | None = None


class EditGroupMember(InteractaModel):
    # creato da rreverse engineering, non supportato da api a settembre 2023
    id: int | None = None
    occ_token: int | None = None
    add_user_ids: list[int] | None = None
    delete_user_ids: list[int] | None = None


class EditGroupMembersIn(InteractaModel):
    # creato da rreverse engineering, non supportato da api a settembre 2023
    group_members: list[EditGroupMember] | None = None


### Catalogs


class GetPostDefinitionCatalogsIn(InteractaModel):
    # GetPostDefinitionCatalogsRequest
    catalog_ids: list[int] = []


### Comments


class ListPostCommentsIn(PaginatedIn):
    # ListPostCommentsRequestDTO
    pass


class InputPostCommentAttachmentIn(InteractaModel):
    # InputPostCommentAttachmentDTO
    attachment_id: int | None = None
    name: str | None = None
    content_ref: str | None = None


class CreatePostCommentIn(InteractaModel):
    # CreatePostCommentRequestDTO
    comment: str | None = None
    comment_format: int = 1  # Formato del commento, facoltativo (1=delta, 2=plainText, default: 1)
    client_uid: str | None = None
    attachments: InputPostCommentAttachmentIn | None = None
    parent_comment_id: int | None = None



from typing import Any

from pydantic import BaseModel, ConfigDict, PrivateAttr
from pydantic.alias_generators import to_camel


class InteractaModel(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        validate_assignment=True,
    )

    def get_absolute_url(self, base_url: str | None = None) -> str:
        raise NotImplementedError


class InteractaOut(InteractaModel):
    _response: Any = PrivateAttr(None)


class PaginatedOut(InteractaOut):
    items: list | None = []
    next_page_token: str | None = None
    total_items_count: int | None = None

    def count(self):
        if not self.items:
            return 0
        return len(self.items)

    def has_items(self):
        if self.count():
            return True
        return False


class ItemCreatedEditedOut(InteractaOut):
    next_occ_token: int | None = None


class PaginatedIn(InteractaModel):
    page_token: str | None = None
    page_size: int = 15
    calculate_total_items_count: bool | None = True


class InteractaIn(PaginatedIn):
    order_desc: bool | None = None
